import express from 'express';
import { Logger } from 'winston';
import * as graphql_modules from 'graphql-modules';
import { Module } from 'graphql-modules';
import { Entity } from '@backstage/catalog-model';
import * as _envelop_core from '@envelop/core';
import { envelop } from '@envelop/core';
import DataLoader from 'dataloader';
import { CatalogClient } from '@backstage/catalog-client';
import * as graphql from 'graphql';
import { TypeSource } from '@graphql-tools/utils';

declare type EntityRef = string | {
    kind: string;
    namespace?: string;
    name: string;
};
declare type Loader = DataLoader<EntityRef, Entity>;
declare type EnvelopPlugins = Parameters<typeof envelop>[0]['plugins'];

declare type createGraphQLAppOptions<Plugins extends EnvelopPlugins, Loader extends DataLoader<any, any>> = {
    loader: () => Loader;
    plugins?: Plugins;
    modules?: Module[];
    refToId?: (ref: EntityRef) => string;
};
declare function createGraphQLApp<Plugins extends EnvelopPlugins, Loader extends DataLoader<any, any>>(options: createGraphQLAppOptions<Plugins, Loader>): {
    run: _envelop_core.GetEnvelopedFn<{
        [x: string]: any;
        [x: number]: any;
        [x: symbol]: any;
    }>;
    application: graphql_modules.Application;
};

declare function createLoader(catalog: Pick<CatalogClient, 'getEntitiesByRefs'>): Loader;

declare function transformSchema(source: TypeSource): graphql.GraphQLSchema;

declare type RouterOptions = {
    logger: Logger;
    catalog: CatalogClient;
    modules?: Module[];
    plugins?: EnvelopPlugins;
};
declare function createRouter(options: RouterOptions): Promise<express.Router>;

export { Loader as EntityLoader, RouterOptions, createGraphQLApp, createGraphQLAppOptions, createLoader, createRouter, transformSchema };
